@@ -43,6 +43,8 @@ class ConfigManager:
         self._log_loaded_config()
         self._initialize_prompts()
         self._ensure_entity_extraction_defaults()
+        self._ensure_retriever_defaults()
+        self._normalize_vector_store_config()
 
     def _log_loaded_config(self) -> None:
         """Emit a debug log with the loaded configuration."""
@@ -90,7 +92,9 @@ class ConfigManager:
                 "works with": "AFFILIATED_WITH",
             },
         )
-        entity_config.setdefault("graph_database_config", {"output_file": "graph_output.json"})
+        entity_config.setdefault(
+            "graph_database_config", {"output_file": "graph_output.json"}
+        )
         parsing_defaults = entity_config.setdefault("relationship_parsing", {})
         parsing_defaults.setdefault("preferred_format", "auto")
         parsing_defaults.setdefault("parser_class", None)
@@ -101,11 +105,107 @@ class ConfigManager:
         prompt_defaults.setdefault("default", "relationship_extraction")
         prompt_defaults.setdefault("providers", {})
 
-        retriever_defaults = entity_config.setdefault("graph_retriever", {})
-        retriever_defaults.setdefault("enabled", False)
-        retriever_defaults.setdefault("backend", "simple")
-        retriever_defaults.setdefault("top_k", 5)
-        retriever_defaults.setdefault("include_edges", True)
+    def _ensure_retriever_defaults(self) -> None:
+        """
+        Ensure defaults exist for the unified retriever configuration.
+
+        This creates a comprehensive retriever configuration with:
+        - Vector retrieval settings
+        - Graph retrieval settings
+        - Hybrid combination strategy
+        """
+        retriever_config = self._config.setdefault("retriever", {})
+
+        # Vector retrieval defaults
+        vector_config = retriever_config.setdefault("vector", {})
+        vector_config.setdefault("enabled", True)
+        vector_config.setdefault("top_k", 3)
+        vector_config.setdefault("search_type", "similarity")
+        vector_config.setdefault("search_kwargs", {})
+
+        # Graph retrieval defaults
+        graph_config = retriever_config.setdefault("graph", {})
+        graph_config.setdefault("enabled", True)
+        graph_config.setdefault("backend", "networkx")
+        graph_config.setdefault("top_k", 5)
+        graph_config.setdefault("max_hops", 2)
+        graph_config.setdefault("include_edges", True)
+        graph_config.setdefault("traversal_strategy", "bfs")
+        graph_config.setdefault("min_score", 0.0)
+
+        # Hybrid combination defaults
+        hybrid_config = retriever_config.setdefault("hybrid", {})
+        hybrid_config.setdefault("mode", "concat")
+        hybrid_config.setdefault("vector_weight", 0.6)
+        hybrid_config.setdefault("graph_weight", 0.4)
+        hybrid_config.setdefault("deduplicate", True)
+
+    def _normalize_vector_store_config(self) -> None:
+        """
+        Support both legacy ``vector_store`` and the newer ``vector_stores`` schema.
+
+        The new format lets users define multiple backends and select a default. We
+        normalize it here so downstream code can continue to reference the legacy
+        ``vector_store`` block without caring which syntax the user chose.
+        """
+
+        if "vector_store" in self._config:
+            return
+
+        block = self._config.get("vector_stores")
+        normalized = self._coerce_vector_store_block(block)
+        if normalized:
+            self._config["vector_store"] = normalized
+        elif block is not None:
+            self.logger.warning(
+                "Unable to normalize 'vector_stores' configuration. Falling back to defaults."
+            )
+
+    def _coerce_vector_store_block(
+        self, block: Dict[str, Any] | None
+    ) -> Dict[str, Any] | None:
+        if not isinstance(block, dict):
+            return None
+
+        stores = block.get("stores") or {}
+        default_store = block.get("default_store")
+        if not default_store and stores:
+            default_store = next(iter(stores))
+
+        store_entry = stores.get(default_store) if default_store else None
+        if store_entry is None:
+            return None
+
+        enabled = block.get("enabled")
+        params: Dict[str, Any] = {}
+        store_type = default_store
+
+        if isinstance(store_entry, dict):
+            store_type = store_entry.get("store_type", store_type)
+            if "enabled" in store_entry:
+                enabled = store_entry["enabled"]
+
+            nested_params = store_entry.get("params")
+            values = {
+                key: value
+                for key, value in store_entry.items()
+                if key not in {"store_type", "enabled", "params"}
+            }
+            params.update(values)
+            if isinstance(nested_params, dict):
+                params.update(nested_params)
+        else:
+            store_type = store_entry or store_type
+
+        top_level_params = block.get("params")
+        if isinstance(top_level_params, dict):
+            params.update(top_level_params)
+
+        return {
+            "enabled": True if enabled is None else bool(enabled),
+            "store_type": store_type,
+            "params": params,
+        }
 
     def _load_config(self) -> Dict[str, Any]:
         """Load configuration from YAML and environment overrides."""
@@ -133,7 +233,9 @@ class ConfigManager:
         # Backfill OpenAI API keys when omitted.
         openai_section = embeddings.get("openai")
         if isinstance(openai_section, dict):
-            api_key = openai_section.get("api_key") or openai_section.get("openai_api_key")
+            api_key = openai_section.get("api_key") or openai_section.get(
+                "openai_api_key"
+            )
             if not api_key:
                 openai_section["api_key"] = os.environ.get("OPENAI_API_KEY")
 
@@ -192,7 +294,9 @@ class ConfigManager:
     def entity_extraction_config(self) -> EntityExtractionConfig:
         """Typed entity extraction configuration."""
 
-        return EntityExtractionConfig.model_validate(self._config.get("entity_extraction", {}))
+        return EntityExtractionConfig.model_validate(
+            self._config.get("entity_extraction", {})
+        )
 
     @property
     def graph_database_config(self) -> GraphDatabaseConfig:
@@ -208,91 +312,27 @@ class ConfigManager:
 
         return LLMPromptsConfig.model_validate(self._config.get("llm_prompts", {}))
 
-    def get_loader_mapping(self) -> Dict[str, Type]:
+    def get_loader_mapping(self) -> Dict[str, Type | str]:
         """
-        Get the loader mapping with imported classes.
+        Get the loader mapping with deferred imports.
 
         Returns:
-            Dictionary mapping file extensions to loader classes.
+            Dictionary mapping file extensions to loader classes or import paths.
         """
 
-        # import registry lazily to avoid circular imports at module import time
-        from ragdoll.ingestion import get_loader as registry_get_loader
-
         loaders_config: LoadersConfig | None = self.ingestion_config.loaders
-        result: Dict[str, Type] = {}
+        result: Dict[str, Type | str] = {}
         file_mappings = getattr(loaders_config, "file_mappings", None)
-        self.logger.info("Raw file mappings from config: %s", file_mappings or "None")
+        self.logger.info(
+            "Raw file mappings from config:\n%s",
+            json.dumps(file_mappings or {}, indent=2),
+        )
 
         if file_mappings:
             for ext, class_path in file_mappings.items():
-                self.logger.debug("Loading loader for extension '%s' via %s", ext, class_path)
-                module_path: str | None = None
-
-                try:
-                    # First try the short-name registry (preferred)
-                    loader_class = registry_get_loader(class_path)
-                    if loader_class:
-                        self.logger.debug("Resolved loader for %s via registry: %s", ext, class_path)
-                        try:
-                            from ragdoll.ingestion import register_loader_class
-
-                            norm_ext = ext.lstrip(".").lower() if isinstance(ext, str) else ext
-                            register_loader_class(norm_ext, loader_class)
-                        except Exception:
-                            pass
-                    else:
-                        module_path, class_name = class_path.rsplit(".", 1)
-                        self.logger.debug("Importing module %s, class %s", module_path, class_name)
-                        module = import_module(module_path)
-
-                        if not hasattr(module, class_name):
-                            self.logger.warning(
-                                "Module %s does not have attribute %s for extension %s. Skipping.",
-                                module_path,
-                                class_name,
-                                ext,
-                            )
-                            continue
-
-                        loader_class = getattr(module, class_name)
-                        try:
-                            from ragdoll.ingestion import register_loader_class
-
-                            norm_ext = ext.lstrip(".").lower() if isinstance(ext, str) else ext
-                            register_loader_class(norm_ext, loader_class)
-                        except Exception:
-                            pass
-                    self.logger.info(
-                        "For extension %s: loaded %s from %s",
-                        ext,
-                        loader_class.__name__,
-                        loader_class.__module__,
-                    )
-                    result[ext] = loader_class
-
-                except ImportError:
-                    self.logger.warning(
-                        "Module %s for extension %s could not be imported. "
-                        "This extension will not be supported.",
-                        module_path or class_path,
-                        ext,
-                    )
-                except (AttributeError, ValueError) as exc:
-                    self.logger.warning(
-                        "Error loading loader for extension %s: %s",
-                        ext,
-                        exc,
-                    )
-
-        if result:
-            self.logger.info("Loaded %s file extension loaders.", len(result))
-            for ext, loader in result.items():
-                self.logger.info(
-                    "Final loader mapping: %s -> %s from %s",
-                    ext,
-                    loader.__name__,
-                    loader.__module__,
+                result[ext] = class_path
+                self.logger.debug(
+                    "Registered loader path for extension '%s': %s", ext, class_path
                 )
         else:
             self.logger.warning("No file extension loaders were registered.")
@@ -328,7 +368,9 @@ class ConfigManager:
                     try:
                         from ragdoll.ingestion import register_loader_class
 
-                        norm_ext = ext.lstrip(".").lower() if isinstance(ext, str) else ext
+                        norm_ext = (
+                            ext.lstrip(".").lower() if isinstance(ext, str) else ext
+                        )
                         register_loader_class(norm_ext, loader_class)
                     except Exception:
                         pass
@@ -477,4 +519,3 @@ class ConfigManager:
 
         log_lines.append("")  # Final newline
         return "\n".join(log_lines)
-
